import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// 解决在 MaterialApp 中使用 CupertinoAlertDialog 时出现的 showTimePicker function results in error: The getter 'modalBarrierDismissLabel' was called on null 的 bug
///
/// class MyApp extends StatelessWidget {
///   RouterUtil routeObserver = RouterUtil.getInstance();
///
///   @override
///   Widget build(BuildContext context) {
///     return MaterialApp(
///       localizationsDelegates: [
///       // 在 MyApp 加入这句
///         FallbackCupertinoLocalisationsDelegate(),
///       ],
///       home: MyHomePage(title: 'Home Page'),
///     );
///   }
/// }
/// */

class FallbackCupertinoLocalisationsDelegate extends LocalizationsDelegate<CupertinoLocalizations> {
  const FallbackCupertinoLocalisationsDelegate();

  @override
  bool isSupported(Locale locale) => true;

  @override
  Future<CupertinoLocalizations> load(Locale locale) => DefaultCupertinoLocalizations.load(locale);

  @override
  bool shouldReload(FallbackCupertinoLocalisationsDelegate old) => false;
}

enum AsDropDownDirection { top, bottom, left, right }
// targetKey 目标 widget key
// sizeChild 弹窗的大小
/*
    showDialogAsDropDown(
      context,
      keyTarget,
      Size(30, 50),
      (buildContext) {
        return Container(key: keyChild, width: 30, height: 50, color: Colors.blue, child: Text("AAA"), alignment: Alignment.center);
      },
      direction: AsDropDownDirection.top,
      marginTargetWidget: 5,
      barrierDismissible: true,
    );
* */
Future<T?> showDialogAsDropDown<T>(BuildContext buildContext, GlobalKey targetKey, Size sizeChild, WidgetBuilder childBuilder, {AsDropDownDirection direction = AsDropDownDirection.bottom, int marginTargetWidget = 0, bool barrierDismissible = true}) {
  RenderBox? boxTarget = targetKey.currentContext?.findRenderObject() as RenderBox?;
  Size sizeTarget = boxTarget?.size ?? Size(0, 0);
  Offset offsetTarget = boxTarget?.localToGlobal(Offset.zero) ?? Offset.zero;
  var statusBarHeight = MediaQuery.of(buildContext).padding.top;
  double top = 0, left = 0;
  switch (direction) {
    case AsDropDownDirection.top:
      top = offsetTarget.dy - sizeChild.height - statusBarHeight - marginTargetWidget;
      left = offsetTarget.dx + (sizeTarget.width - sizeChild.width) / 2;
      break;
    case AsDropDownDirection.bottom:
      top = offsetTarget.dy + sizeTarget.height + marginTargetWidget - statusBarHeight;
      left = offsetTarget.dx + (sizeTarget.width - sizeChild.width) / 2;
      break;
    case AsDropDownDirection.left:
      top = offsetTarget.dy - statusBarHeight + (sizeTarget.height - sizeChild.height) / 2;
      left = offsetTarget.dx - sizeChild.width - marginTargetWidget;
      break;
    case AsDropDownDirection.right:
      top = offsetTarget.dy - statusBarHeight + (sizeTarget.height - sizeChild.height) / 2;
      left = offsetTarget.dx + sizeTarget.width + marginTargetWidget;
      break;
  }
  return showDialog<T>(
      context: buildContext,
      barrierColor: Colors.transparent,
      builder: (context) {
        return Dialog(
          backgroundColor: Colors.transparent,
          insetPadding: EdgeInsets.zero,
          elevation: 0,
          child: Stack(
            children: [
              GestureDetector(
                  onTap: () {
                    if (barrierDismissible) Navigator.pop(context);
                  },
                  behavior: HitTestBehavior.translucent,
                  child: Container(width: double.infinity, height: double.infinity)),
              Positioned(top: top, left: left, child: childBuilder.call(context)),
            ],
          ),
        );
      });
}

class CommonDialog extends StatelessWidget {
  static const EdgeInsets _defaultInsetPadding = EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0);
  final Color? backgroundColor;
  final double? elevation;
  final Duration insetAnimationDuration;
  final Curve insetAnimationCurve;
  final EdgeInsets insetPadding;
  final Clip clipBehavior;
  final ShapeBorder? shape;
  final Widget? child;

  const CommonDialog({
    Key? key,
    this.backgroundColor,
    this.elevation,
    this.insetAnimationDuration = const Duration(milliseconds: 100),
    this.insetAnimationCurve = Curves.decelerate,
    this.insetPadding = _defaultInsetPadding,
    this.clipBehavior = Clip.none,
    this.shape,
    this.child,
  })  : assert(clipBehavior != null),
        super(key: key);

  static const RoundedRectangleBorder _defaultDialogShape = RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)));
  static const double _defaultElevation = 24.0;

  @override
  Widget build(BuildContext context) {
    final DialogTheme dialogTheme = DialogTheme.of(context);
    final EdgeInsets effectivePadding = MediaQuery.of(context).viewInsets + (insetPadding);
    return AnimatedPadding(
      padding: effectivePadding,
      duration: insetAnimationDuration,
      curve: insetAnimationCurve,
      child: MediaQuery.removeViewInsets(
        removeLeft: true,
        removeTop: true,
        removeRight: true,
        removeBottom: true,
        context: context,
        child: Center(
          child: ConstrainedBox(
            constraints: const BoxConstraints(),
            child: Material(
              color: backgroundColor ?? dialogTheme.backgroundColor ?? Theme.of(context).dialogBackgroundColor,
              elevation: elevation ?? dialogTheme.elevation ?? _defaultElevation,
              shape: shape ?? dialogTheme.shape ?? _defaultDialogShape,
              type: MaterialType.card,
              clipBehavior: clipBehavior,
              child: child,
            ),
          ),
        ),
      ),
    );
  }
}
