import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// 城市｜时间选择器
typedef OnCityPickerSelectChanged<T extends IPickerViewData<T>> = Function(T province, T city, T count);

class CityPicker<T extends IPickerViewData<T>> extends StatefulWidget {
  final List<T> dataList;
  final int? currentProvinceIndex; //当前所选的县区级别的index
  final int? currentCityIndex; //当前所选的县区级别的index
  final int? currentCountyIndex;
  final OnCityPickerSelectChanged<T>? selectChanged;

  const CityPicker({Key? key, required this.dataList, this.currentProvinceIndex, this.currentCityIndex, this.currentCountyIndex, this.selectChanged}) : super(key: key); //当前所选的县区级别的index

  @override
  _CityPickerState<T> createState() => _CityPickerState<T>();
}

class _CityPickerState<T extends IPickerViewData<T>> extends State<CityPicker<T>> {
  List<T> _provinceList = <T>[];
  List<T> _cityList = <T>[];
  List<T> _countyList = <T>[];
  late FixedExtentScrollController _provinceController;
  late FixedExtentScrollController _cityController;
  late FixedExtentScrollController _countyController;

  @override
  void initState() {
    List<T> dataList = widget.dataList;
    _provinceList.addAll(dataList);
    List<T> cityList = _provinceList[widget.currentProvinceIndex ?? 0].iChildBeanList;
    _cityList.addAll(cityList);
    List<T> countyList = _cityList[widget.currentCityIndex ?? 0].iChildBeanList;
    _countyList.addAll(countyList);
    _provinceController = FixedExtentScrollController(initialItem: widget.currentProvinceIndex ?? 0)
      ..addListener(() {
        _cityList.clear();
        _countyList.clear();
        List<T>? cityListTemp = _provinceList[_provinceController.selectedItem].iChildBeanList;
        if (cityListTemp != null) {
          _cityList.addAll(cityListTemp);
        }
        List<T>? countyListTemp = cityListTemp[0].iChildBeanList;
        if (countyListTemp != null) {
          _countyList.addAll(countyListTemp);
        }
        _cityController.jumpToItem(0);
        _countyController.jumpToItem(0);
        if (mounted) setState(() {});
        T provinceList = _provinceList[_provinceController.selectedItem];
        widget.selectChanged?.call(provinceList, _cityList[0], _countyList[0]);
      });
    _cityController = FixedExtentScrollController(initialItem: widget.currentCityIndex ?? 0)
      ..addListener(() {
        _countyList.clear();
        List<T> countyListTemp = _cityList[_cityController.selectedItem].iChildBeanList;
        _countyList.addAll(countyListTemp);
        _countyController.jumpToItem(0);
        if (mounted) setState(() {});
        widget.selectChanged?.call(_provinceList[_provinceController.selectedItem], _cityList[_countyController.selectedItem], _countyList[0]);
      });
    _countyController = FixedExtentScrollController(initialItem: widget.currentCountyIndex ?? 0)
      ..addListener(() {
        widget.selectChanged?.call(_provinceList[_provinceController.selectedItem], _cityList[_countyController.selectedItem], _countyList[_countyController.selectedItem]);
      });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _provinceController, children: _provinceList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        ),
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _cityController, children: _cityList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        ),
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _countyController, children: _countyList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        )
      ],
    );
  }
}

abstract class IPickerViewData<T extends IPickerViewData<T>> {
  String get iDisplayText;

  List<T> get iChildBeanList;
}

class CupertinoYearPicker extends StatefulWidget {
  final int startYear, endYear;
  final int? currentYear;
  final Function(int index, int selectValue)? onSelect;

  const CupertinoYearPicker(this.startYear, this.endYear, {this.currentYear, this.onSelect, Key? key}) : super(key: key);

  @override
  _CupertinoYearPickerState createState() => _CupertinoYearPickerState();
}

class _CupertinoYearPickerState extends State<CupertinoYearPicker> {
  List<int> _list = [];
  FixedExtentScrollController? _cupertinoPickerController;

  @override
  void initState() {
    var generate = List.generate(widget.endYear - widget.startYear+1, (index) => (widget.startYear + index));
    _list.addAll(generate);
    var indexOf = _list.indexOf(widget.currentYear ?? 0, 0);
    if (indexOf <= 0) indexOf = 0;
    _cupertinoPickerController = FixedExtentScrollController(initialItem: indexOf);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return CupertinoPicker(
      selectionOverlay: null,
      itemExtent: 32,
      scrollController: _cupertinoPickerController,
      onSelectedItemChanged: (i) => widget.onSelect?.call(i, _list[i]),
      children: _list.map((e) => Text(e.toString(), maxLines: 1, style: TextStyle(fontSize: 18, color: Colors.black.withOpacity(0.8)))).toList(),
    );
  }

  @override
  void dispose() {
    _cupertinoPickerController?.dispose();
    super.dispose();
  }
}

class CupertinoMonthPicker extends StatefulWidget {
  final DateTime startDate, endDate;
  final DateTime? currentDate;
  final Function(int indexYear, int indexMonth, DateTime selectValue)? onSelect;

  const CupertinoMonthPicker(this.startDate, this.endDate, {this.currentDate, this.onSelect, Key? key}) : super(key: key);

  @override
  _CupertinoMonthPickerState createState() => _CupertinoMonthPickerState();
}

class _CupertinoMonthPickerState extends State<CupertinoMonthPicker> {
  List<int> _listYear = [];
  List<int> _listMonth = [];
  FixedExtentScrollController? _yearPickerController, _monthPickerController;

  @override
  void initState() {
    var generate = List.generate(widget.endDate.year - widget.startDate.year + 1, (index) => (widget.startDate.year + index));
    _listYear.addAll(generate);
    var indexYear = _listYear.indexOf(widget.currentDate?.year ?? 0, 0);
    if (indexYear <= 0) indexYear = 0;

    _listMonth = _setMonthRange(indexYear);

    var indexMonth = _listMonth.indexOf(widget.currentDate?.month ?? 0, 0);
    _yearPickerController = FixedExtentScrollController(initialItem: indexYear);
    _monthPickerController = FixedExtentScrollController(initialItem: indexMonth);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Row(children: [
      Expanded(
        child: CupertinoPicker(
          itemExtent: 32,
          selectionOverlay: null,
          onSelectedItemChanged: (i) {
            setState(() => _listMonth = _setMonthRange(i));
            widget.onSelect?.call(i, _monthPickerController?.selectedItem ?? 0, DateTime(_listYear[i], (_monthPickerController?.selectedItem ?? 0) + 1));
          },
          scrollController: _yearPickerController,
          children: _listYear.map((e) => Text(e.toString(), maxLines: 1, style: TextStyle(fontSize: 18, color: Colors.black.withOpacity(0.8)))).toList(),
        ),
      ),
      Expanded(
        child: CupertinoPicker(
          itemExtent: 32,
          selectionOverlay: null,
          onSelectedItemChanged: (i) => widget.onSelect?.call(_yearPickerController?.selectedItem ?? 0, i, DateTime(_listYear[_yearPickerController?.selectedItem ?? 0], i + 1)),
          looping: true,
          scrollController: _monthPickerController,
          children: _listMonth.map((e) => Text(e.toString(), maxLines: 1, style: TextStyle(fontSize: 18, color: Colors.black.withOpacity(0.8)))).toList(),
        ),
      ),
    ]);
  }

  List<int> _setMonthRange(int selectYearIndex) {
    if (selectYearIndex == 0 && widget.startDate.month != 1) {
      var month = widget.startDate.month;
      var list = List.generate(12 - month + 1, (index) => index + month);
      _monthPickerController?.jumpTo(0);
      return list;
    }
    if (selectYearIndex == _listYear.length - 1 && widget.endDate.month != 12) {
      var month = widget.endDate.month;
      var list = List.generate(month, (index) => index + 1);
      _monthPickerController?.jumpTo(0);
      return list;
    }
    return List.generate(12, (index) => index + 1);
  }

  @override
  void dispose() {
    _yearPickerController?.dispose();
    _monthPickerController?.dispose();
    super.dispose();
  }
}

class CupertinoYearRangePicker extends StatefulWidget {
  final int startYear, endYear;
  final String startStr, endStr;
  final Function(int selectStart, int selectEnd)? onSelect;

  const CupertinoYearRangePicker(this.startYear, this.endYear, {this.startStr = "开始日期", this.endStr = "结束日期", this.onSelect, Key? key}) : super(key: key);

  @override
  _CupertinoYearRangePickerState createState() => _CupertinoYearRangePickerState();
}

class _CupertinoYearRangePickerState extends State<CupertinoYearRangePicker> {
  var _start = 0, _end = 0;
  final style = TextStyle(fontSize: 14, color: Colors.black.withOpacity(0.8));

  @override
  void initState() {
    _start = widget.startYear;
    _end = widget.endYear;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Row(children: [Expanded(child: Center(child: Text(widget.startStr, style: style))), Expanded(child: Center(child: Text(widget.endStr, style: style)))]),
        Expanded(
          child: Row(
            children: [
              Expanded(
                  child: CupertinoYearPicker(widget.startYear, widget.endYear, currentYear: _start, onSelect: (i, v) {
                _start = v;
                widget.onSelect?.call(_start, _end);
              })),
              Expanded(
                  child: CupertinoYearPicker(widget.startYear, widget.endYear, currentYear: _end, onSelect: (i, v) {
                _end = v;
                widget.onSelect?.call(_start, _end);
              })),
            ],
          ),
        )
      ],
    );
  }
}

class CupertinoMonthRangePicker extends StatefulWidget {
  final DateTime startDate, endDate;
  final String startStr, endStr;
  final Function(DateTime selectStart, DateTime selectEnd)? onSelect;

  CupertinoMonthRangePicker(this.startDate, this.endDate, {this.startStr = "开始日期", this.endStr = "结束日期", this.onSelect, Key? key}) : super(key: key);

  @override
  _CupertinoMonthRangePickerState createState() => _CupertinoMonthRangePickerState();
}

class _CupertinoMonthRangePickerState extends State<CupertinoMonthRangePicker> {
  final style = TextStyle(fontSize: 14, color: Colors.black.withOpacity(0.8));
  late DateTime _start, _end;

  @override
  void initState() {
    _start = widget.startDate;
    _end = widget.endDate;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Row(children: [Expanded(child: Center(child: Text(widget.startStr, style: style))), Expanded(child: Center(child: Text(widget.endStr, style: style)))]),
        Expanded(
          child: Row(
            children: [
              Expanded(
                  child: CupertinoMonthPicker(widget.startDate, widget.endDate, currentDate: widget.startDate, onSelect: (y, m, d) {
                _start = d;
                widget.onSelect?.call(_start, _end);
              })),
              Expanded(
                  child: CupertinoMonthPicker(widget.startDate, widget.endDate, currentDate: widget.endDate, onSelect: (y, m, d) {
                _end = d;
                widget.onSelect?.call(_start, _end);
              })),
            ],
          ),
        )
      ],
    );
  }
}
